@@ -238,7 +238,7 @@ static int cmd_df(int argc, char **argv)
 	}
 	ret = get_df(fd, &sargs);
 
-	if (!ret && sargs) {
+	if (ret == 0) {
 		print_df(sargs);
 		free(sargs);
 	} else {
@@ -513,7 +513,7 @@ static int dev_to_fsid(char *dev, __u8 *fsid)
 
 	disk_super = (struct btrfs_super_block *)buf;
 	ret = btrfs_read_dev_super(fd, disk_super,
-			BTRFS_SUPER_INFO_OFFSET);
+				   BTRFS_SUPER_INFO_OFFSET, 0);
 	if (ret)
 		goto out;
 
@@ -547,7 +547,7 @@ static int cmd_show(int argc, char **argv)
 	char mp[BTRFS_PATH_NAME_MAX + 1];
 	char path[PATH_MAX];
 	__u8 fsid[BTRFS_FSID_SIZE];
-	char uuid_buf[37];
+	char uuid_buf[BTRFS_UUID_UNPARSED_SIZE];
 	int found = 0;
 
 	while (1) {
@@ -656,7 +656,6 @@ devs_only:
 	while (!list_empty(all_uuids)) {
 		fs_devices = list_entry(all_uuids->next,
 					struct btrfs_fs_devices, list);
-		list_del(&fs_devices->list);
 		btrfs_close_devices(fs_devices);
 	}
 out:
@@ -924,10 +923,11 @@ static int cmd_defrag(int argc, char **argv)
 }
 
 static const char * const cmd_resize_usage[] = {
-	"btrfs filesystem resize [devid:][+/-]<newsize>[gkm]|[devid:]max <path>",
+	"btrfs filesystem resize [devid:][+/-]<newsize>[kKmMgGtTpPeE]|[devid:]max <path>",
 	"Resize a filesystem",
 	"If 'max' is passed, the filesystem will occupy all available space",
 	"on the device 'devid'.",
+	"[kK] means KiB, which denotes 1KiB = 1024B, 1MiB = 1024KiB, etc.",
 	NULL
 };
 
