@@ -129,7 +129,10 @@ static int find_good_parent(struct btrfs_send *s, u64 root_id, u64 *found)
 		parent2 = subvol_uuid_search(&s->sus, s->clone_sources[i], NULL,
 				0, NULL, subvol_search_by_root_id);
 
-		assert(parent2);
+		if (!parent2) {
+			ret = -ENOENT;
+			goto out;
+		}
 		tmp = parent2->ctransid - parent->ctransid;
 		if (tmp < 0)
 			tmp *= -1;
@@ -346,9 +349,17 @@ static int init_root_path(struct btrfs_send *s, const char *subvol)
 
 	ret = find_mount_root(subvol, &s->root_path);
 	if (ret < 0) {
+		fprintf(stderr,
+			"ERROR: failed to determine mount point for %s: %s\n",
+			subvol, strerror(-ret));
+		ret = -EINVAL;
+		goto out;
+	}
+	if (ret > 0) {
+		fprintf(stderr,
+			"ERROR: %s doesn't belong to btrfs mount point\n",
+			subvol);
 		ret = -EINVAL;
-		fprintf(stderr, "ERROR: failed to determine mount point "
-				"for %s\n", subvol);
 		goto out;
 	}
 
@@ -519,7 +530,7 @@ int cmd_send(int argc, char **argv)
 		}
 	}
 
-	if (optind == argc)
+	if (check_argc_min(argc - optind, 1))
 		usage(cmd_send_usage);
 
 	if (outname != NULL) {
@@ -583,6 +594,13 @@ int cmd_send(int argc, char **argv)
 				strerror(-ret));
 			goto out;
 		}
+		if (ret > 0) {
+			fprintf(stderr,
+			"ERROR: %s doesn't belong to btrfs mount point\n",
+				subvol);
+			ret = -EINVAL;
+			goto out;
+		}
 		if (strcmp(send.root_path, mount_root) != 0) {
 			ret = -EINVAL;
 			fprintf(stderr, "ERROR: all subvols must be from the "
